from typing import List

from .exception import *
from .waifu import Waifu
from .func import *

def GenerateWaifu(seeds: list=None, step=0)-> Waifu:
    # There is no way to get only one waifu, so we generate 16 and pick the first one
    return GenerateWaifus(seeds, step)[0] 

def GenerateWaifus(seeds: list=None, step=0) -> List[Waifu]:
    """
        Generate 16 waifus

        `seeds`: Generate a waifu based on a waifu seed
    """

    # Getting ready to send a resp to the API
    waifu_obj = {} # The waifu object to send to the api.
    waifus = [] # All the waifu's the API sends back to us.
    waifu_obj['step'] = max(0, min(3, step)) # Sets the steps. Makes sure it is in the range of 0 - 3
    
    if waifu_obj['step'] > 0: # Checks if we are in step 1, 2, or 3
        waifu_obj['currentGirl'] = seeds # Adds the seed to the waifu object to be sent.
        if not isvalidseed(waifu_obj['currentGirl']): # Checks if the seed is a valid seed that can be sent.
            raise WaifuInvalidSeed("No valid 'Waifu' or 'Seeds' provided")
    
    resp = valid_response(fetch("generate", waifu_obj)) # Everything is ok! Send the request.

    if "newGirls" not in resp: # Checking if we did not get any new waifus
        raise WaifuInvalidResponse(f"Expected 'newGirls' in resp, got {resp}.")

    generatedWaifus = resp['newGirls'] # Get the new waifus
    
    for x in generatedWaifus: # Make a new Waifu object for all waifus generated
        waifus.append(Waifu(base64=x['image'], seeds=x['seeds']))

    return waifus


import requests
import aiohttp
import json
from .exception import WaifuInvalidResponse

baseURL = "https://api.waifulabs.com/"

session = requests.session()
aio_session = aiohttp.ClientSession(headers={"Content-Type": "application/json"})

PILLOW = "PILLOW"
POSTER = "POSTER"

_non_200_err = "Got a non 200 HTTP code. Received {}."

def fetch(endpoint, data):
    return session.post(baseURL + endpoint, json=data, headers={
        "Content-Type": "application/json"
    })

async def fetch_async(endpoint, data):
    r = await aio_session.post(baseURL + endpoint, json=data)

    return r

def valid_response(response: requests.Response):
    if response.status_code != 200:
        raise WaifuInvalidResponse(_non_200_err.format(response.status_code))

    return response.json()

async def valid_async_response(response: aiohttp.ClientResponse):
    if response.status != 200:
        raise WaifuInvalidResponse(_non_200_err.format(response.status))

    t = await response.text()

    return json.loads(t)

def isvalidseed(seeds: list):
    if len(seeds) < 17: return False
    for seed in seeds[0:16]:
        if type(seed) != int:
            return False

    return True


import base64
import pathlib

from .exception import *
from .func import *

class Waifu:
    """
        Represents a Waifulabs Waifu
        
        `base64`: str -> Base64 image of the waifu

        `seeds`: list -> The waifu's seeds

        `big`: bool -> See if the waifu is of type `big`

        `product`: bool -> See if the waifu is of type `product`

        `product_type`: str -> If waifu is product, see product type
    """
    def __init__(self, **kwargs):
        self.base64 = ""
        self.seeds = []
        self.big = False
        self.product = False
        self.product_type = False
        if "base64" in kwargs:
            if not type(kwargs['base64']) == str:
                raise TypeError(f"Argument 'base64' was type {type(kwargs['base64'])}, but expected {str}")

            self.base64 = kwargs['base64']
            self.bytes = base64.decodebytes(kwargs["base64"].encode())
        else:
            raise WaifuMissingArg("Arg 'base64' is missing!")

        if "seeds" in kwargs:
            if not type(kwargs['seeds']) == list:
                raise TypeError(f"Argument 'seeds' was type {type(kwargs['seeds'])}, but expected {list}")

            self.seeds = kwargs['seeds']
        else:
            raise WaifuMissingArg("Arg 'seeds' is missing!")

        if "big" in kwargs:
            if not type(kwargs['big']) == bool:
                raise TypeError(f"Argument 'big' was type {type(kwargs['big'])}, but expected {bool}")

            self.big = kwargs['big']

        if "product" in kwargs:
            if not type(kwargs['product']) == bool:
                raise TypeError(f"Argument 'product' was type {type(kwargs['product'])}, but expected {bool}")

            self.product = kwargs['product']

            if not "product_type" in kwargs:
                raise WaifuMissingArg("Arg 'product_type' is required when arg 'product' is set.")

            if not type(kwargs['product_type']) == str:
                raise TypeError(f"Argument 'product_type' was type {type(kwargs['product_type'])}, but expected {str}")

            self.product_type = kwargs['product_type']

    def _exists(self, file: str):
        return pathlib.Path(file).exists()
    
    def save(self, file: str) -> None:
        """
            Save a image of the current waifu.

            `file`: Filename. Directories allowed.
        """
        if not self._exists(file):
            p = file.split("/")
            del p[-1]
            pathlib.Path("/".join(p)).mkdir(parents=True, exist_ok=True)

        with open(file, "wb") as f:
            f.write(self.bytes)

    def GenerateBigWaifu(self) -> "Waifu":
        """
            Generate a big waifu based on the current waifu
        """
        if not isvalidseed(self.seeds):
            raise WaifuInvalidSeed("No valid 'Waifu' or 'Seeds' provided")
        
        bigwaifu = valid_response(fetch("generate_big", {"currentGirl": self.seeds}))

        return Waifu(base64=bigwaifu['girl'], seeds=self.seeds, big=True)

    def GenerateProduct(self, product: str) -> "Waifu":
        """
            Generate a product based on the current waifu
        """
        if not isvalidseed(self.seeds):
            raise WaifuInvalidSeed("No valid 'Waifu' or 'Seeds' provided")

        if product != PILLOW or product != POSTER:
            raise WaifuInvalidProduct(f"Got {product}, not waifulabs.PILLOW, or waifulabs.POSTER")

        waifuproduct = valid_response(fetch("generate_preview", {
            "currentGirl": self.seeds,
            "product": product
        }))

        return Waifu(base64=waifuproduct['girl'], seeds=self.seeds, product=True, product_type=product)

class AsyncWaifu(Waifu):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    async def GenerateBigWaifu(self) -> "AsyncWaifu":
        """
            Generate a big waifu based on the current waifu
        """
        if not isvalidseed(self.seeds):
            raise WaifuInvalidSeed("No valid 'Waifu' or 'Seeds' provided")
        
        resp = await fetch("generate_big", {"currentGirl": self.seeds})
        
        bigwaifu = await valid_async_response(resp)

        return AsyncWaifu(base64=bigwaifu['girl'], seeds=self.seeds, big=True)

    async def GenerateProduct(self, product: str) -> "AsyncWaifu":
        """
            Generate a product based on the current waifu
        """
        if not isvalidseed(self.seeds):
            raise WaifuInvalidSeed("No valid 'Waifu' or 'Seeds' provided")

        if product != PILLOW or product != POSTER:
            raise WaifuInvalidProduct(f"Got {product}, not waifulabs.PILLOW, or waifulabs.POSTER")

        resp = await fetch_async("generate_preview", {
            "currentGirl": self.seeds,
            "product": product
        })

        waifuproduct = await valid_response(resp)

        return AsyncWaifu(base64=waifuproduct['girl'], seeds=self.seeds, product=True, product_type=product)


# -*- coding: utf-8 -*-

"""
Unofficial Waifulabs API Wrapper

A Waifulabs wrapper to generate Waifu's with AI.

:copyright: (c) 2021 TaromaruYuki
:license: MIT, see LICENSE for more details.
"""

__title__ = "Waifulabs"
__author__ = "TaromaruYuki"
__license__ = "MIT"
__copyright__ = "Copyright 2021 TaromaruYuki"
__version__ = "1.1.0"

from .asynchronous import *
from .exception import *
from .sync import *
from .waifu import *



import setuptools
import waifulabs

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name=waifulabs.__title__,
    version=waifulabs.__version__,
    author=waifulabs.__author__,
    author_email="taromaruyuki@gmail.com",
    description="A unofficial wrapper for WaifuLabs",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Taromaruu/WaifuLabs",
    download_url="https://github.com/Taromaruu/WaifuLabs/releases/",
    install_requires=["requests", "aiohttp"],
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 5 - Production/Stable"
    ],
    python_requires='>=3.6'
)